import {
  Controller,
  Get,
  Post,
  Body,
  Patch,
  Param,
  Delete,
  HttpCode,
  HttpStatus,
  Query,
  Res,
} from '@nestjs/common';
import { ApiTags, ApiOperation, ApiResponse, ApiParam, ApiQuery } from '@nestjs/swagger';
import { ProjectsService } from '../services/projects.service';
import { CreateProjectDto } from '../dto/create-project.dto';
import { UpdateProjectDto } from '../dto/update-project.dto';

@ApiTags('projects')
@Controller('projects')
export class ProjectsController {
  constructor(private readonly projectsService: ProjectsService) {}

  /**
   * Create a new project
   */
  @ApiOperation({ summary: 'Create a new project' })
  @ApiResponse({ status: 201, description: 'The project has been successfully created.' })
  @ApiResponse({ status: 400, description: 'Bad request.' })
  @Post()
  @HttpCode(HttpStatus.CREATED)
  create(@Body() createProjectDto: CreateProjectDto) {
    return this.projectsService.create(createProjectDto);
  }

  /**
   * Get all projects or filter by owner ID
   */
  @ApiOperation({ summary: 'Get all projects or filter by owner ID' })
  @ApiResponse({ status: 200, description: 'Return all projects or projects for a specific owner.' })
  @ApiQuery({ name: 'ownerId', required: false, description: 'Filter projects by owner ID' })
  @Get()
  findAll(@Query('ownerId') ownerId?: string) {
    if (ownerId) {
      return this.projectsService.findByOwnerId(ownerId);
    }
    return this.projectsService.findAll();
  }

  /**
   * Get a project by ID
   */
  @ApiOperation({ summary: 'Get a project by ID' })
  @ApiResponse({ status: 200, description: 'Return the project.' })
  @ApiResponse({ status: 404, description: 'Project not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.projectsService.findById(id);
  }

  /**
   * Update a project
   */
  @ApiOperation({ summary: 'Update a project' })
  @ApiResponse({ status: 200, description: 'The project has been successfully updated.' })
  @ApiResponse({ status: 400, description: 'Bad request.' })
  @ApiResponse({ status: 404, description: 'Project not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @Patch(':id')
  update(@Param('id') id: string, @Body() updateProjectDto: UpdateProjectDto) {
    return this.projectsService.update(id, updateProjectDto);
  }

  /**
   * Delete a project
   */
  @ApiOperation({ summary: 'Delete a project' })
  @ApiResponse({ status: 204, description: 'The project has been successfully deleted.' })
  @ApiResponse({ status: 404, description: 'Project not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @Delete(':id')
  @HttpCode(HttpStatus.NO_CONTENT)
  remove(@Param('id') id: string) {
    return this.projectsService.remove(id);
  }

  /**
   * Check if a user has access to a project
   */
  @ApiOperation({ summary: 'Check if a user has access to a project' })
  @ApiResponse({ status: 200, description: 'Returns true if the user has access, false otherwise.' })
  @ApiResponse({ status: 404, description: 'Project not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @ApiParam({ name: 'userId', description: 'The ID of the user' })
  @Get(':id/check-access/:userId')
  async checkUserAccess(
    @Param('id') id: string, 
    @Param('userId') userId: string,
    @Res() res: any
  ) {
    const hasAccess = await this.projectsService.checkUserAccess(id, userId);
    // Send the boolean value directly as the response body
    res.json(hasAccess);
  }

  /**
   * Add a collaborator to a project
   */
  @ApiOperation({ summary: 'Add a collaborator to a project' })
  @ApiResponse({ status: 201, description: 'The collaborator has been successfully added to the project.' })
  @ApiResponse({ status: 404, description: 'Project or user not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @ApiParam({ name: 'userId', description: 'The ID of the user to add as a collaborator' })
  @Post(':id/collaborators/:userId')
  @HttpCode(HttpStatus.CREATED)
  addCollaborator(@Param('id') id: string, @Param('userId') userId: string) {
    return this.projectsService.addCollaborator(id, userId);
  }

  /**
   * Remove a collaborator from a project
   */
  @ApiOperation({ summary: 'Remove a collaborator from a project' })
  @ApiResponse({ status: 204, description: 'The collaborator has been successfully removed from the project.' })
  @ApiResponse({ status: 404, description: 'Project or collaborator not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @ApiParam({ name: 'userId', description: 'The ID of the user to remove as a collaborator' })
  @Delete(':id/collaborators/:userId')
  @HttpCode(HttpStatus.NO_CONTENT)
  removeCollaborator(@Param('id') id: string, @Param('userId') userId: string) {
    return this.projectsService.removeCollaborator(id, userId);
  }

  /**
   * Get all collaborators for a project
   */
  @ApiOperation({ summary: 'Get all collaborators for a project' })
  @ApiResponse({ status: 200, description: 'Return all collaborators for the project.' })
  @ApiResponse({ status: 404, description: 'Project not found.' })
  @ApiParam({ name: 'id', description: 'The ID of the project' })
  @Get(':id/collaborators')
  getCollaborators(@Param('id') id: string) {
    return this.projectsService.getCollaborators(id);
  }
}
