"use client";

import { useState } from "react";
import Link from "next/link";
import { Button } from "@/components/ui/button";
import { Input } from "@/components/ui/input";
import { 
  Table, 
  TableBody, 
  TableCell, 
  TableHead, 
  TableHeader, 
  TableRow 
} from "@/components/ui/table";
import { 
  DropdownMenu, 
  DropdownMenuContent, 
  DropdownMenuItem, 
  DropdownMenuLabel, 
  DropdownMenuSeparator, 
  DropdownMenuTrigger 
} from "@/components/ui/dropdown-menu";
import { 
  PlusCircle, 
  Search, 
  MoreHorizontal, 
  Pencil, 
  Trash2, 
  Users,
  CircleDot
} from "lucide-react";
import { Badge } from "@/components/ui/badge";
import { toast } from "sonner";

export default function AdminTagsPage() {
  const [searchQuery, setSearchQuery] = useState("");

  // Mock data for global tags
  const tags = [
    {
      id: 1,
      name: "Frontend",
      description: "Développement frontend",
      color: "blue",
      usageCount: 12,
      global: true,
      createdBy: "Admin",
    },
    {
      id: 2,
      name: "Backend",
      description: "Développement backend",
      color: "green",
      usageCount: 8,
      global: true,
      createdBy: "Admin",
    },
    {
      id: 3,
      name: "Fullstack",
      description: "Développement fullstack",
      color: "purple",
      usageCount: 5,
      global: true,
      createdBy: "Admin",
    },
    {
      id: 4,
      name: "UX/UI",
      description: "Design UX/UI",
      color: "pink",
      usageCount: 3,
      global: true,
      createdBy: "Marie Martin",
    },
    {
      id: 5,
      name: "DevOps",
      description: "Infrastructure et déploiement",
      color: "orange",
      usageCount: 2,
      global: true,
      createdBy: "Thomas Bernard",
    },
    {
      id: 6,
      name: "Junior",
      description: "Niveau junior",
      color: "yellow",
      usageCount: 7,
      global: true,
      createdBy: "Admin",
    },
    {
      id: 7,
      name: "Medior",
      description: "Niveau intermédiaire",
      color: "amber",
      usageCount: 5,
      global: true,
      createdBy: "Admin",
    },
    {
      id: 8,
      name: "Senior",
      description: "Niveau senior",
      color: "red",
      usageCount: 6,
      global: true,
      createdBy: "Admin",
    },
  ];

  // Map color names to Tailwind classes
  const colorMap: Record<string, string> = {
    blue: "bg-blue-100 text-blue-800 dark:bg-blue-900 dark:text-blue-300",
    green: "bg-green-100 text-green-800 dark:bg-green-900 dark:text-green-300",
    purple: "bg-purple-100 text-purple-800 dark:bg-purple-900 dark:text-purple-300",
    pink: "bg-pink-100 text-pink-800 dark:bg-pink-900 dark:text-pink-300",
    orange: "bg-orange-100 text-orange-800 dark:bg-orange-900 dark:text-orange-300",
    yellow: "bg-yellow-100 text-yellow-800 dark:bg-yellow-900 dark:text-yellow-300",
    amber: "bg-amber-100 text-amber-800 dark:bg-amber-900 dark:text-amber-300",
    red: "bg-red-100 text-red-800 dark:bg-red-900 dark:text-red-300",
  };

  // Filter tags based on search query
  const filteredTags = tags.filter(
    (tag) =>
      tag.name.toLowerCase().includes(searchQuery.toLowerCase()) ||
      tag.description.toLowerCase().includes(searchQuery.toLowerCase()) ||
      tag.createdBy.toLowerCase().includes(searchQuery.toLowerCase())
  );

  const handleDeleteTag = (tagId: number) => {
    toast.success(`Tag #${tagId} supprimé avec succès`);
  };

  return (
    <div className="flex flex-col gap-6">
      <div className="flex flex-col sm:flex-row sm:items-center sm:justify-between gap-4">
        <h1 className="text-2xl sm:text-3xl font-bold">Tags globaux</h1>
        <Button asChild className="w-full sm:w-auto">
          <Link href="/admin/tags/new">
            <PlusCircle className="mr-2 h-4 w-4" />
            Nouveau tag global
          </Link>
        </Button>
      </div>

      <div className="flex items-center gap-2">
        <div className="relative flex-1">
          <Search className="absolute left-2.5 top-2.5 h-4 w-4 text-muted-foreground" />
          <Input
            type="search"
            placeholder="Rechercher des tags..."
            className="pl-8"
            value={searchQuery}
            onChange={(e) => setSearchQuery(e.target.value)}
          />
        </div>
      </div>

      {/* Mobile card view */}
      <div className="grid gap-4 sm:hidden">
        {filteredTags.length === 0 ? (
          <div className="rounded-md border p-6 text-center text-muted-foreground">
            Aucun tag trouvé.
          </div>
        ) : (
          filteredTags.map((tag) => (
            <div key={tag.id} className="rounded-md border p-4">
              <div className="flex items-center justify-between">
                <Badge className={colorMap[tag.color]}>
                  {tag.name}
                </Badge>
                <Badge variant="outline" className="text-xs">
                  {tag.usageCount} utilisations
                </Badge>
              </div>
              <div className="mt-2">
                <p className="text-sm text-muted-foreground">{tag.description}</p>
              </div>
              <div className="mt-2 text-xs text-muted-foreground">
                Créé par: {tag.createdBy}
              </div>
              <div className="mt-4 flex justify-end gap-2">
                <Button variant="outline" size="sm" asChild>
                  <Link href={`/admin/tags/${tag.id}/edit`}>
                    <Pencil className="mr-2 h-4 w-4" />
                    Modifier
                  </Link>
                </Button>
                <Button
                  variant="ghost"
                  size="sm"
                  onClick={() => handleDeleteTag(tag.id)}
                  className="text-destructive hover:text-destructive"
                >
                  <Trash2 className="h-4 w-4" />
                </Button>
              </div>
            </div>
          ))
        )}
      </div>

      {/* Desktop table view */}
      <div className="rounded-md border hidden sm:block overflow-auto">
        <Table>
          <TableHeader>
            <TableRow>
              <TableHead>Tag</TableHead>
              <TableHead>Description</TableHead>
              <TableHead>Utilisations</TableHead>
              <TableHead>Créé par</TableHead>
              <TableHead className="w-[100px]">Actions</TableHead>
            </TableRow>
          </TableHeader>
          <TableBody>
            {filteredTags.length === 0 ? (
              <TableRow>
                <TableCell colSpan={5} className="h-24 text-center">
                  Aucun tag trouvé.
                </TableCell>
              </TableRow>
            ) : (
              filteredTags.map((tag) => (
                <TableRow key={tag.id}>
                  <TableCell>
                    <Badge className={colorMap[tag.color]}>
                      {tag.name}
                    </Badge>
                  </TableCell>
                  <TableCell>{tag.description}</TableCell>
                  <TableCell>{tag.usageCount}</TableCell>
                  <TableCell>{tag.createdBy}</TableCell>
                  <TableCell>
                    <DropdownMenu>
                      <DropdownMenuTrigger asChild>
                        <Button variant="ghost" size="icon">
                          <MoreHorizontal className="h-4 w-4" />
                          <span className="sr-only">Actions</span>
                        </Button>
                      </DropdownMenuTrigger>
                      <DropdownMenuContent align="end">
                        <DropdownMenuLabel>Actions</DropdownMenuLabel>
                        <DropdownMenuSeparator />
                        <DropdownMenuItem asChild>
                          <Link href={`/admin/tags/${tag.id}/edit`} className="flex items-center">
                            <Pencil className="mr-2 h-4 w-4" />
                            <span>Modifier</span>
                          </Link>
                        </DropdownMenuItem>
                        <DropdownMenuItem asChild>
                          <Link href={`/admin/tags/${tag.id}/usage`} className="flex items-center">
                            <Users className="mr-2 h-4 w-4" />
                            <span>Voir les utilisations</span>
                          </Link>
                        </DropdownMenuItem>
                        <DropdownMenuItem
                          onClick={() => handleDeleteTag(tag.id)}
                          className="text-destructive focus:text-destructive flex items-center"
                        >
                          <Trash2 className="mr-2 h-4 w-4" />
                          <span>Supprimer</span>
                        </DropdownMenuItem>
                      </DropdownMenuContent>
                    </DropdownMenu>
                  </TableCell>
                </TableRow>
              ))
            )}
          </TableBody>
        </Table>
      </div>
    </div>
  );
}